

import java.math.BigDecimal;

import com.improvess.selecpos.model.Avaliacao;
import com.improvess.selecpos.model.NotaConceitual;

public class DadosExperienciaDocente {

	private static final BigDecimal[] NOTAS_TERCEIRO_GRAU_AREA_AFIM = new BigDecimal[] {
			Avaliacao.DEZ, Avaliacao.NOVE, Avaliacao.OITO, Avaliacao.SEIS,
			Avaliacao.CINCO, Avaliacao.ZERO };
	private static final BigDecimal[] NOTAS_MONITORIA = new BigDecimal[] {
			Avaliacao.QUATRO, Avaliacao.TRES, Avaliacao.DOIS, Avaliacao.UM,
			Avaliacao.ZERO_VIRGULA_CINCO, Avaliacao.ZERO };
	private static final BigDecimal[] NOTAS_ASSITENTE = new BigDecimal[] {
			Avaliacao.QUATRO, Avaliacao.TRES, Avaliacao.DOIS, Avaliacao.UM,
			Avaliacao.ZERO_VIRGULA_CINCO, Avaliacao.ZERO };

	private NotaConceitual terceiroGrauAreaAfim;
	private NotaConceitual monitoria;
	private NotaConceitual assistenteLaboratorio;
	private String justificativaExperienciaDocente;
	private BigDecimal notaSomatorioExperienciaDocencente;
	private BigDecimal notaCalculadaExperienciaDocencente;
	private boolean notaEditada;

	public void reset() {
		this.terceiroGrauAreaAfim = null;
		this.monitoria = null;
		this.assistenteLaboratorio = null;
		this.justificativaExperienciaDocente = null;
		this.notaSomatorioExperienciaDocencente = null;
		this.notaCalculadaExperienciaDocencente = null;
		this.notaEditada = false;
	}

	private void calcularNota() {
		BigDecimal notaTerceiroGrauAreaAfim;
		if (this.terceiroGrauAreaAfim != null) {
			notaTerceiroGrauAreaAfim = this.calcularNotaConceitual(
					this.terceiroGrauAreaAfim, NOTAS_TERCEIRO_GRAU_AREA_AFIM);
		} else {
			notaTerceiroGrauAreaAfim = Avaliacao.ZERO;
		}
		BigDecimal notaMonitoria;
		if (this.monitoria != null) {
			notaMonitoria = this.calcularNotaConceitual(this.monitoria,
					NOTAS_MONITORIA);
		} else {
			notaMonitoria = Avaliacao.ZERO;
		}
		BigDecimal notaAssistenteLaboratorio;
		if (this.assistenteLaboratorio != null) {
			notaAssistenteLaboratorio = this.calcularNotaConceitual(
					this.assistenteLaboratorio, NOTAS_ASSITENTE);
		} else {
			notaAssistenteLaboratorio = Avaliacao.ZERO;
		}
		this.notaSomatorioExperienciaDocencente = notaTerceiroGrauAreaAfim.add(
				notaMonitoria).add(notaAssistenteLaboratorio);
		if (this.notaSomatorioExperienciaDocencente.compareTo(Avaliacao.DEZ) > 0) {
			this.notaCalculadaExperienciaDocencente = Avaliacao.DEZ;
		} else {
			this.notaCalculadaExperienciaDocencente = this.notaSomatorioExperienciaDocencente;
		}
	}

	private BigDecimal calcularNotaConceitual(NotaConceitual notaConceitual,
			BigDecimal[] valores) {
		BigDecimal retorno;
		switch (notaConceitual) {
		case MAIOR_IGUAL_TRES_A:
			retorno = valores[0];
			break;
		case DOIS_A:
			retorno = valores[1];
			break;
		case UM_VIRGULA_CONCO_A:
			retorno = valores[2];
			break;
		case UM_A:
			retorno = valores[3];
			break;
		case MEIO_A:
			retorno = valores[4];
			break;
		case NAO:
			retorno = valores[5];
			break;
		default:
			throw new RuntimeException("no implementado");
		}
		return retorno;
	}

	public void setTerceiroGrauAreaAfim(NotaConceitual terceiroGrauAreaAfim) {
		this.terceiroGrauAreaAfim = terceiroGrauAreaAfim;
	}

	public NotaConceitual getTerceiroGrauAreaAfim() {
		return terceiroGrauAreaAfim;
	}

	public NotaConceitual getMonitoria() {
		return monitoria;
	}

	public void setMonitoria(NotaConceitual monitoria) {
		this.monitoria = monitoria;
	}

	public NotaConceitual getAssistenteLaboratorio() {
		return assistenteLaboratorio;
	}

	public void setAssistenteLaboratorio(NotaConceitual assistenteLaboratorio) {
		this.assistenteLaboratorio = assistenteLaboratorio;
	}

	public String getJustificativaExperienciaDocente() {
		return justificativaExperienciaDocente;
	}

	public void setJustificativaExperienciaDocente(
			String justificativaExperienciaDocente) {
		this.justificativaExperienciaDocente = justificativaExperienciaDocente;
	}

	public void setNotaSomatorioExperienciaDocencente(
			BigDecimal notaSomatorioExperienciaDocencente) {
		this.notaSomatorioExperienciaDocencente = notaSomatorioExperienciaDocencente;
	}

	public BigDecimal getNotaSomatorioExperienciaDocencente() {
		if (!this.notaEditada) {
			this.calcularNota();
		}
		return notaSomatorioExperienciaDocencente;
	}

	public void setNotaCalculadaExperienciaDocencente(
			BigDecimal notaCalculadaExperienciaDocencente) {
		this.notaCalculadaExperienciaDocencente = notaCalculadaExperienciaDocencente
				.setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
	}

	public BigDecimal getNotaCalculadaExperienciaDocencente() {
		if (!this.notaEditada) {
			this.calcularNota();
		}
		return notaCalculadaExperienciaDocencente;
	}

	public void setAvaliacao(Avaliacao avaliacao) {
		this.terceiroGrauAreaAfim = avaliacao.getTerceiroGrauAreaAfim();
		this.monitoria = avaliacao.getMonitoria();
		this.assistenteLaboratorio = avaliacao.getAssistenteLaboratorio();
		this.justificativaExperienciaDocente = avaliacao
				.getJustificativaExperienciaDocente();
		this.notaSomatorioExperienciaDocencente = avaliacao
				.getNotaSomatorioExperienciaDocencente();
		this.notaCalculadaExperienciaDocencente = avaliacao
				.getNotaCalculadaExperienciaDocencente();
		if (avaliacao.getNotaExperienciaDocencenteEditada() != null) {
			this.notaEditada = avaliacao.getNotaExperienciaDocencenteEditada();
		} else {
			this.notaEditada = false;
		}
	}

	public boolean getNotaEditada() {
		return notaEditada;
	}

	public void setNotaEditada(boolean notaEditada) {
		this.notaEditada = notaEditada;
	}
}
