

import java.math.BigDecimal;

import com.improvess.selecpos.model.Avaliacao;
import com.improvess.selecpos.model.NotaConceitual;

public class DadosExperienciaPeDeI {

	private static final BigDecimal[] NOTAS_INICIACAO_CIENTIFICA = new BigDecimal[] {
			Avaliacao.NOVE, Avaliacao.OITO, Avaliacao.SETE, Avaliacao.SEIS,
			Avaliacao.CINCO, Avaliacao.ZERO };
	private static final BigDecimal[] NOTAS_BOLSA = new BigDecimal[] {
			Avaliacao.DEZ, Avaliacao.NOVE, Avaliacao.OITO, Avaliacao.SETE,
			Avaliacao.SEIS, Avaliacao.ZERO };
	private NotaConceitual iniciacaoCientifica;
	private NotaConceitual bolsaPeDeI;

	private String justificativaPeDeI;

	private BigDecimal notaSomatorioPeDeI;
	private BigDecimal notaCalculadaPeDeI;
	private boolean notaEditada;

	public void reset() {
		this.iniciacaoCientifica = null;
		this.bolsaPeDeI = null;

		this.justificativaPeDeI = null;

		this.notaSomatorioPeDeI = null;
		this.notaCalculadaPeDeI = null;
		this.notaEditada = false;
	}

	private void calcularNota() {
		BigDecimal notaIniciacaoCientifica;
		if (this.iniciacaoCientifica != null) {
			notaIniciacaoCientifica = this.calcularNotaConceitual(
					this.iniciacaoCientifica, NOTAS_INICIACAO_CIENTIFICA);
		} else {
			notaIniciacaoCientifica = Avaliacao.ZERO;
		}
		BigDecimal notaBolsa;
		if (this.bolsaPeDeI != null) {
			notaBolsa = this.calcularNotaConceitual(this.bolsaPeDeI,
					NOTAS_BOLSA);
		} else {
			notaBolsa = Avaliacao.ZERO;
		}
		this.notaSomatorioPeDeI = notaIniciacaoCientifica.add(notaBolsa);

		if (this.notaSomatorioPeDeI.compareTo(Avaliacao.DEZ) < 0) {
			this.notaCalculadaPeDeI = this.notaSomatorioPeDeI;
		} else {
			this.notaCalculadaPeDeI = Avaliacao.DEZ;
		}
	}

	private BigDecimal calcularNotaConceitual(NotaConceitual notaConceitual,
			BigDecimal[] valores) {
		BigDecimal retorno;
		switch (notaConceitual) {
		case MAIOR_IGUAL_TRES_A:
			retorno = valores[0];
			break;
		case DOIS_A:
			retorno = valores[1];
			break;
		case UM_VIRGULA_CONCO_A:
			retorno = valores[2];
			break;
		case UM_A:
			retorno = valores[3];
			break;
		case MEIO_A:
			retorno = valores[4];
			break;
		case NAO:
			retorno = valores[5];
			break;
		default:
			throw new RuntimeException("no implementado");
		}
		return retorno;
	}

	public BigDecimal getNotaSomatorioPeDeI() {
		if (!this.notaEditada) {
			this.calcularNota();
		}
		return this.notaSomatorioPeDeI;
	}

	public void setNotaSomatorioPeDeI(BigDecimal notaSomatorioPeDeI) {
		this.notaSomatorioPeDeI = notaSomatorioPeDeI;
	}

	public BigDecimal getNotaCalculadaPeDeI() {
		if (!this.notaEditada) {
			this.calcularNota();
		}
		return this.notaCalculadaPeDeI;
	}

	public void setNotaCalculadaPeDeI(BigDecimal notaCalculadaPeDeI) {
		this.notaCalculadaPeDeI = notaCalculadaPeDeI.setScale(Avaliacao.SCALE,
				Avaliacao.ROUND_MODE);
	}

	public void setIniciacaoCientifica(NotaConceitual iniciacaoCientifica) {
		this.iniciacaoCientifica = iniciacaoCientifica;
	}

	public NotaConceitual getIniciacaoCientifica() {
		return iniciacaoCientifica;
	}

	public String getJustificativaPeDeI() {
		return justificativaPeDeI;
	}

	public void setJustificativaPeDeI(String justificativaPeDeI) {
		this.justificativaPeDeI = justificativaPeDeI;
	}

	public void setBolsaPeDeI(NotaConceitual bolsaPeDeI) {
		this.bolsaPeDeI = bolsaPeDeI;
	}

	public NotaConceitual getBolsaPeDeI() {
		return bolsaPeDeI;
	}

	public void setAvaliacao(Avaliacao avaliacao) {
		this.iniciacaoCientifica = avaliacao.getInciacaoCientifica();
		this.bolsaPeDeI = avaliacao.getBolsaPeDeI();

		this.justificativaPeDeI = avaliacao.getJustificativaPeDeI();

		this.notaSomatorioPeDeI = avaliacao.getNotaSomatorioPeDeI();
		this.notaCalculadaPeDeI = avaliacao.getNotaCalculadaPeDeI();
		if (avaliacao.getNotaPeDeIEditada() != null) {
			this.notaEditada = avaliacao.getNotaPeDeIEditada();
		} else {
			this.notaEditada = false;
		}
	}

	public boolean getNotaEditada() {
		return notaEditada;
	}

	public void setNotaEditada(boolean notaEditada) {
		this.notaEditada = notaEditada;
	}
}
