

import java.math.BigDecimal;

import com.improvess.selecpos.model.Avaliacao;
import com.improvess.selecpos.model.NotaConceitual;

public class DadosExperienciaProfissionalNaoDocente {

	private static final BigDecimal[] NOTAS_SUPERIOR_VINCULO_AREA_COORDENACAO = new BigDecimal[] {
			Avaliacao.DEZ, Avaliacao.OITO, Avaliacao.SEIS, Avaliacao.QUATRO,
			Avaliacao.DOIS, Avaliacao.ZERO };
	private static final BigDecimal[] NOTAS_SUPERIOR_VINCULO_AREA = new BigDecimal[] {
			Avaliacao.OITO, Avaliacao.SEIS, Avaliacao.QUATRO, Avaliacao.DOIS,
			Avaliacao.UM, Avaliacao.ZERO };
	private static final BigDecimal[] NOTAS_MEDIO_VINCULO_AREA = new BigDecimal[] {
			Avaliacao.CINCO, Avaliacao.QUATRO, Avaliacao.TRES, Avaliacao.DOIS,
			Avaliacao.UM, Avaliacao.ZERO };
	private static final BigDecimal[] NOTAS_SUPERIOR_SEM_VINCULO = new BigDecimal[] {
			Avaliacao.OITO, Avaliacao.SEIS, Avaliacao.QUATRO, Avaliacao.DOIS,
			Avaliacao.UM, Avaliacao.ZERO };
	private static final BigDecimal[] NOTAS_ESTAGIO_ESTRACURRICULAR = new BigDecimal[] {
			Avaliacao.SEIS, Avaliacao.CINCO, Avaliacao.QUATRO, Avaliacao.TRES,
			Avaliacao.DOIS, Avaliacao.ZERO };
	private static final BigDecimal[] NOTAS_PROFISSIONAL_AREA_DISTINTA = new BigDecimal[] {
			Avaliacao.TRES, Avaliacao.DOIS, Avaliacao.UM, Avaliacao.ZERO,
			Avaliacao.ZERO, Avaliacao.ZERO };

	private NotaConceitual superiorComVinculoEmpregaticioNaAreaCoodenando;
	private NotaConceitual superiorComVinculoEmpregaticioNaArea;
	private NotaConceitual medioComVinculoEmpregaticioNaArea;
	private NotaConceitual superiorSemVinculo;
	private NotaConceitual estagioExtraCurricular;
	private NotaConceitual profissionalAreaDistinta;

	private String justificativaExperienciaNaoDocente;

	private BigDecimal notaSomatorioExperienciaNaoDocente;
	private BigDecimal notaCalculadaExperienciaNaoDocente;
	private boolean notaEditada;

	public void reset() {
		this.superiorComVinculoEmpregaticioNaAreaCoodenando = null;
		this.superiorComVinculoEmpregaticioNaArea = null;
		this.medioComVinculoEmpregaticioNaArea = null;
		this.superiorSemVinculo = null;
		this.estagioExtraCurricular = null;
		this.profissionalAreaDistinta = null;

		this.justificativaExperienciaNaoDocente = null;

		this.notaSomatorioExperienciaNaoDocente = null;
		this.notaCalculadaExperienciaNaoDocente = null;

		this.notaEditada = false;
	}

	private void calcularNota() {

		BigDecimal notaSuperiorComVinculoEmpregaticioNaAreaCoodenando;
		if (superiorComVinculoEmpregaticioNaAreaCoodenando != null) {
			notaSuperiorComVinculoEmpregaticioNaAreaCoodenando = this
					.calcularNotaConceitual(
							superiorComVinculoEmpregaticioNaAreaCoodenando,
							NOTAS_SUPERIOR_VINCULO_AREA_COORDENACAO);
		} else {
			notaSuperiorComVinculoEmpregaticioNaAreaCoodenando = Avaliacao.ZERO;
		}

		BigDecimal notaSuperiorComVinculoEmpregaticioNaArea;
		if (superiorComVinculoEmpregaticioNaArea != null) {
			notaSuperiorComVinculoEmpregaticioNaArea = this
					.calcularNotaConceitual(
							superiorComVinculoEmpregaticioNaArea,
							NOTAS_SUPERIOR_VINCULO_AREA);
		} else {
			notaSuperiorComVinculoEmpregaticioNaArea = Avaliacao.ZERO;
		}
		BigDecimal notaMedioComVinculoEmpregaticioNaArea;
		if (medioComVinculoEmpregaticioNaArea != null) {
			notaMedioComVinculoEmpregaticioNaArea = this
					.calcularNotaConceitual(medioComVinculoEmpregaticioNaArea,
							NOTAS_MEDIO_VINCULO_AREA);
		} else {
			notaMedioComVinculoEmpregaticioNaArea = Avaliacao.ZERO;
		}
		BigDecimal notaSuperiorSemVinculo;
		if (superiorSemVinculo != null) {
			notaSuperiorSemVinculo = this.calcularNotaConceitual(
					superiorSemVinculo, NOTAS_SUPERIOR_SEM_VINCULO);
		} else {
			notaSuperiorSemVinculo = Avaliacao.ZERO;
		}
		BigDecimal notaEstagioExtraCurricular;
		if (estagioExtraCurricular != null) {
			notaEstagioExtraCurricular = this.calcularNotaConceitual(
					estagioExtraCurricular, NOTAS_ESTAGIO_ESTRACURRICULAR);
		} else {
			notaEstagioExtraCurricular = Avaliacao.ZERO;
		}
		BigDecimal notaProfissionalAreaDistinta;
		if (profissionalAreaDistinta != null) {
			notaProfissionalAreaDistinta = this.calcularNotaConceitual(
					profissionalAreaDistinta, NOTAS_PROFISSIONAL_AREA_DISTINTA);
		} else {
			notaProfissionalAreaDistinta = Avaliacao.ZERO;
		}
		this.notaSomatorioExperienciaNaoDocente = notaSuperiorComVinculoEmpregaticioNaAreaCoodenando
				.add(notaSuperiorComVinculoEmpregaticioNaArea).add(
						notaMedioComVinculoEmpregaticioNaArea).add(
						notaSuperiorSemVinculo).add(notaEstagioExtraCurricular)
				.add(notaProfissionalAreaDistinta);

		if (this.notaSomatorioExperienciaNaoDocente.compareTo(Avaliacao.DEZ) < 0) {
			this.notaCalculadaExperienciaNaoDocente = this.notaSomatorioExperienciaNaoDocente;
		} else {
			this.notaCalculadaExperienciaNaoDocente = Avaliacao.DEZ;
		}
	}

	private BigDecimal calcularNotaConceitual(NotaConceitual notaConceitual,
			BigDecimal[] valores) {
		BigDecimal retorno;
		switch (notaConceitual) {
		case MAIOR_IGUAL_TRES_A:
			retorno = valores[0];
			break;
		case DOIS_A:
			retorno = valores[1];
			break;
		case UM_VIRGULA_CONCO_A:
			retorno = valores[2];
			break;
		case UM_A:
			retorno = valores[3];
			break;
		case MEIO_A:
			retorno = valores[4];
			break;
		case NAO:
			retorno = valores[5];
			break;
		default:
			throw new RuntimeException("no implementado");
		}
		return retorno;
	}

	public NotaConceitual getSuperiorComVinculoEmpregaticioNaAreaCoodenando() {
		return superiorComVinculoEmpregaticioNaAreaCoodenando;
	}

	public void setSuperiorComVinculoEmpregaticioNaAreaCoodenando(
			NotaConceitual superiorComVinculoEmpregaticioNaAreaCoodenando) {
		this.superiorComVinculoEmpregaticioNaAreaCoodenando = superiorComVinculoEmpregaticioNaAreaCoodenando;
	}

	public NotaConceitual getSuperiorComVinculoEmpregaticioNaArea() {
		return superiorComVinculoEmpregaticioNaArea;
	}

	public void setSuperiorComVinculoEmpregaticioNaArea(
			NotaConceitual superiorComVinculoEmpregaticioNaArea) {
		this.superiorComVinculoEmpregaticioNaArea = superiorComVinculoEmpregaticioNaArea;
	}

	public NotaConceitual getMedioComVinculoEmpregaticioNaArea() {
		return medioComVinculoEmpregaticioNaArea;
	}

	public void setMedioComVinculoEmpregaticioNaArea(
			NotaConceitual medioComVinculoEmpregaticioNaArea) {
		this.medioComVinculoEmpregaticioNaArea = medioComVinculoEmpregaticioNaArea;
	}

	public NotaConceitual getSuperiorSemVinculo() {
		return superiorSemVinculo;
	}

	public void setSuperiorSemVinculo(NotaConceitual superiorSemVinculo) {
		this.superiorSemVinculo = superiorSemVinculo;
	}

	public NotaConceitual getEstagioExtraCurricular() {
		return estagioExtraCurricular;
	}

	public void setEstagioExtraCurricular(NotaConceitual estagioExtraCurricular) {
		this.estagioExtraCurricular = estagioExtraCurricular;
	}

	public NotaConceitual getProfissionalAreaDistinta() {
		return profissionalAreaDistinta;
	}

	public void setProfissionalAreaDistinta(
			NotaConceitual profissionalAreaDistinta) {
		this.profissionalAreaDistinta = profissionalAreaDistinta;
	}

	public String getJustificativaExperienciaNaoDocente() {
		return justificativaExperienciaNaoDocente;
	}

	public void setJustificativaExperienciaNaoDocente(
			String justificativaExperienciaNaoDocente) {
		this.justificativaExperienciaNaoDocente = justificativaExperienciaNaoDocente;
	}

	public void setNotaSomatorioExperienciaNaoDocente(
			BigDecimal notaSomatorioExperienciaNaoDocente) {
		this.notaSomatorioExperienciaNaoDocente = notaSomatorioExperienciaNaoDocente;
	}

	public BigDecimal getNotaSomatorioExperienciaNaoDocente() {
		if (!this.notaEditada) {
			this.calcularNota();
		}
		return this.notaSomatorioExperienciaNaoDocente;
	}

	public BigDecimal getNotaCalculadaExperienciaNaoDocente() {
		if (!this.notaEditada) {
			this.calcularNota();
		}
		return this.notaCalculadaExperienciaNaoDocente;
	}

	public void setNotaCalculadaExperienciaNaoDocente(
			BigDecimal notaCalculadaExperienciaNaoDocente) {
		this.notaCalculadaExperienciaNaoDocente = notaCalculadaExperienciaNaoDocente
				.setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
	}

	public void setAvaliacao(Avaliacao avaliacao) {
		this.superiorComVinculoEmpregaticioNaAreaCoodenando = avaliacao
				.getSuperiorComVinculoEmpregaticioNaAreaCoodenando();
		this.superiorComVinculoEmpregaticioNaArea = avaliacao
				.getSuperiorComVinculoEmpregaticioNaArea();
		this.medioComVinculoEmpregaticioNaArea = avaliacao
				.getMedioComVinculoEmpregaticioNaArea();
		this.superiorSemVinculo = avaliacao.getSuperiorSemVinculo();
		this.estagioExtraCurricular = avaliacao.getEstagioExtraCurricular();
		this.profissionalAreaDistinta = avaliacao.getProfissionalAreaDistinta();

		this.justificativaExperienciaNaoDocente = avaliacao
				.getJustificativaExperienciaNaoDocente();

		this.notaSomatorioExperienciaNaoDocente = avaliacao
				.getNotaSomatorioExperienciaNaoDocente();
		this.notaCalculadaExperienciaNaoDocente = avaliacao
				.getNotaCalculadaExperienciaNaoDocente();
		if (avaliacao.getNotaExperienciaNaoDocenteEditada() != null) {
			this.notaEditada = avaliacao.getNotaExperienciaNaoDocenteEditada();
		} else {
			this.notaEditada = false;
		}
	}

	public boolean getNotaEditada() {
		return notaEditada;
	}
	
	public void setNotaEditada(boolean notaEditada) {
		this.notaEditada = notaEditada;
	}
}
