

import java.math.BigDecimal;

import com.improvess.selecpos.model.Avaliacao;
import com.improvess.selecpos.model.AvaliacaoDoutorado;
import com.improvess.selecpos.model.AvaliacaoMestrado;
import com.improvess.selecpos.model.ConceitoCapes;
import com.improvess.selecpos.model.TipoCurso;

public class DadosHistorico {

	private static final BigDecimal PESO_NOTA_GRADUACAO = Avaliacao.SETE;
	private static final BigDecimal PESO_CAPES = Avaliacao.TRES;
	private static final BigDecimal BONUS_PREMIO = Avaliacao.ZERO_VIRGULA_CINCO;
	private static final BigDecimal BONUS_POS_GRADUACAO = Avaliacao.ZERO_VIRGULA_CINCO;
	private static final BigDecimal PENALIDADA_MESTRADO_ANTIGO = new BigDecimal(
			"0.25").setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
	private static final BigDecimal PESO_HISTORICO_MESTRADO = Avaliacao.QUATRO;
	private static final BigDecimal PESO_NOTA_GRADUACAO_DOUTORADO = Avaliacao.SEIS;

	private BigDecimal notaGraduacao;
	private ConceitoCapes conceitoCapes;
	private boolean temPremios;
	private boolean temPosGraduacaoConcluida;
	private boolean conclusaoMestradoSuperiorLimite;
	private String justificativaNotaHistoricoGraduacao;
	private BigDecimal notaCalculadaHistoricoGraduacao;
	private boolean notaEditada;

	private TipoCurso tipoCurso;
	private BigDecimal notaMestrado;

	public void reset(BigDecimal notaGraduacao) {
		this.notaGraduacao = notaGraduacao;
		this.conceitoCapes = null;
		this.temPremios = false;
		this.temPosGraduacaoConcluida = false;
		this.conclusaoMestradoSuperiorLimite = false;
		this.justificativaNotaHistoricoGraduacao = null;
		this.notaCalculadaHistoricoGraduacao = null;
		this.notaEditada = false;
	}

	private void calcularNota() {
		BigDecimal valorNotaGraduacao = this.notaGraduacao;

		if (this.temPosGraduacaoConcluida) {
			valorNotaGraduacao = valorNotaGraduacao.add(BONUS_POS_GRADUACAO);
		}
		if (this.temPremios) {
			valorNotaGraduacao = valorNotaGraduacao.add(BONUS_PREMIO);
		}

		BigDecimal notaCapes;
		if (this.conceitoCapes != null) {
			notaCapes = this.conceitoCapes.getNota();
		} else {
			notaCapes = Avaliacao.ZERO;
		}

		if (this.tipoCurso == TipoCurso.MESTRADO) {
			this.notaCalculadaHistoricoGraduacao = valorNotaGraduacao.multiply(
					PESO_NOTA_GRADUACAO).add(notaCapes.multiply(PESO_CAPES))
					.divide(Avaliacao.DEZ);
			if (this.notaCalculadaHistoricoGraduacao.compareTo(Avaliacao.DEZ) > 0) {
				this.notaCalculadaHistoricoGraduacao = Avaliacao.DEZ;
			}
		} else {
			BigDecimal valorNotaMestrado = this.notaMestrado;
			if (this.conclusaoMestradoSuperiorLimite) {
				valorNotaMestrado = valorNotaMestrado
						.subtract(PENALIDADA_MESTRADO_ANTIGO);
			}
			valorNotaGraduacao = valorNotaGraduacao.multiply(
					PESO_NOTA_GRADUACAO).add(notaCapes.multiply(PESO_CAPES))
					.divide(Avaliacao.DEZ);
			this.notaCalculadaHistoricoGraduacao = valorNotaGraduacao.multiply(
					PESO_NOTA_GRADUACAO_DOUTORADO).add(
					valorNotaMestrado.multiply(PESO_HISTORICO_MESTRADO)).divide(Avaliacao.DEZ);
			if(this.notaCalculadaHistoricoGraduacao.compareTo(Avaliacao.DEZ) > 0){
				this.notaCalculadaHistoricoGraduacao = Avaliacao.DEZ;
			}
		}
	}

	public ConceitoCapes getConceitoCapes() {
		return conceitoCapes;
	}

	public void setConceitoCapes(ConceitoCapes conceitoCapes) {
		this.conceitoCapes = conceitoCapes;
	}

	public BigDecimal getNotaCalculadaHistoricoGraduacao() {
		if (!this.notaEditada) {
			this.calcularNota();
		}
		return notaCalculadaHistoricoGraduacao;
	}

	public void setNotaCalculadaHistoricoGraduacao(
			BigDecimal notaCalculadaHistoricoGraduacao) {
		this.notaCalculadaHistoricoGraduacao = notaCalculadaHistoricoGraduacao
				.setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
	}

	public boolean getNotaEditada() {
		return notaEditada;
	}

	public void setNotaEditada(boolean notaEditada) {
		this.notaEditada = notaEditada;
	}

	public void setJustificativaNotaHistoricoGraduacao(
			String justificativaNotaHistoricoGraduacao) {
		this.justificativaNotaHistoricoGraduacao = justificativaNotaHistoricoGraduacao;
	}

	public String getJustificativaNotaHistoricoGraduacao() {
		return justificativaNotaHistoricoGraduacao;
	}

	public boolean getConclusaoMestradoSuperiorLimite() {
		return conclusaoMestradoSuperiorLimite;
	}

	public void setConclusaoMestradoSuperiorLimite(
			boolean conclusaoMestradoSuperiorLimite) {
		this.conclusaoMestradoSuperiorLimite = conclusaoMestradoSuperiorLimite;
	}

	public void setTemPosGraduacaoConcluida(boolean temPosGraduacaoConcluida) {
		this.temPosGraduacaoConcluida = temPosGraduacaoConcluida;
	}

	public boolean getTemPosGraduacaoConcluida() {
		return temPosGraduacaoConcluida;
	}

	public void setTemPremios(boolean temPremios) {
		this.temPremios = temPremios;
	}

	public boolean getTemPremios() {
		return temPremios;
	}

	public void setAvaliacaoMestrado(AvaliacaoMestrado avaliacao) {
		setAvaliacao(avaliacao);
	}

	public void setAvaliacaoDoutorado(AvaliacaoDoutorado avaliacao) {
		setAvaliacao(avaliacao);
		this.conclusaoMestradoSuperiorLimite = avaliacao
				.getConclusaoMestradoSuperiorLimite();
	}

	public void setTipoCurso(TipoCurso tipoCurso) {
		this.tipoCurso = tipoCurso;
	}

	private void setAvaliacao(Avaliacao avaliacao) {
		this.conceitoCapes = avaliacao.getConceitoCapes();
		this.temPremios = avaliacao.getTemPremios();
		this.temPosGraduacaoConcluida = avaliacao.getTemPosGraduacaoConcluida();
		this.justificativaNotaHistoricoGraduacao = avaliacao
				.getJustificativaNotaHistoricoGraduacao();
		this.notaCalculadaHistoricoGraduacao = avaliacao
				.getNotaCalculadaHistoricoGraduacao();
		if (avaliacao.getNotaHistoricoGraduacaoEditada() != null) {
			this.notaEditada = avaliacao.getNotaHistoricoGraduacaoEditada();
		} else {
			this.notaEditada = false;
		}
	}

	public void setNotaMestrado(BigDecimal notaMestrado) {
		this.notaMestrado = notaMestrado;
	}
	
	public BigDecimal getNotaMestrado() {
		return notaMestrado;
	}
}
