

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.improvess.selecpos.model.Avaliacao;
import com.improvess.selecpos.model.AvaliacaoPublicacao;
import com.improvess.selecpos.model.ExtratoQualis;
import com.improvess.selecpos.model.TipoPublicacao;

public class DadosProducaoCientificaTecnologica {

	private static final int QUANTIDADE_INICIAL_PUBLICACOES = 5;

	private List<AvaliacaoPublicacao> avaliacoesPublicacoes;
	private String justificativaProducaoCientifica;
	private BigDecimal notaSomatorioProducaoCientifica;
	private BigDecimal notaCalculadaProducaoCientifica;


	private int proximaLinhaPublicacao;

	private boolean notaEditada;

	public void reset() {
		this.avaliacoesPublicacoes = null;
		this.justificativaProducaoCientifica = null;
		this.notaSomatorioProducaoCientifica = null;
		this.notaCalculadaProducaoCientifica = null;
		this.notaEditada = false;
	}

	private void calcularNota() {
		if (this.avaliacoesPublicacoes != null) {
			this.notaSomatorioProducaoCientifica = this.calcularNota(this.avaliacoesPublicacoes);
		} else {
			this.notaSomatorioProducaoCientifica = Avaliacao.ZERO;
		}

		if (this.notaSomatorioProducaoCientifica.compareTo(Avaliacao.DEZ) > 0) {
			this.notaCalculadaProducaoCientifica = Avaliacao.DEZ;
		} else {
			this.notaCalculadaProducaoCientifica = this.notaSomatorioProducaoCientifica;
		}
	}

	public int getProximaLinhaPublicacao() {
		int retorno = proximaLinhaPublicacao;
		proximaLinhaPublicacao++;
		return retorno;
	}

	public void adicionarPublicacao() {
		this.avaliacoesPublicacoes.add(new AvaliacaoPublicacao());
	}

	public List<AvaliacaoPublicacao> getAvaliacoesPublicacoes() {
		this.proximaLinhaPublicacao = 0;
		if (this.avaliacoesPublicacoes == null) {
			List<AvaliacaoPublicacao> retorno = new ArrayList<AvaliacaoPublicacao>();
			this.avaliacoesPublicacoes = retorno;
			for (int i = 0; i < QUANTIDADE_INICIAL_PUBLICACOES; i++) {
				this.adicionarPublicacao();
			}
		}

		return this.avaliacoesPublicacoes;
	}

	public void setAvaliacoesPublicacoes(
			List<AvaliacaoPublicacao> avaliacoesPublicacoes) {
		this.avaliacoesPublicacoes = avaliacoesPublicacoes;
	}

	public BigDecimal getNotaSomatorioProducaoCientifica() {
		if (!this.notaEditada) {
			this.calcularNota();
		}
		return this.notaSomatorioProducaoCientifica;
	}

	public void setNotaSomatorioProducaoCientifica(
			BigDecimal notaSomatorioProducaoCientifica) {
		this.notaSomatorioProducaoCientifica = notaSomatorioProducaoCientifica;
	}

	public void setNotaCalculadaProducaoCientifica(
			BigDecimal notaCalculadaProducaoCientifica) {
		this.notaCalculadaProducaoCientifica = notaCalculadaProducaoCientifica
				.setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
	}

	public BigDecimal getNotaCalculadaProducaoCientifica() {
		if (!this.notaEditada) {
			this.calcularNota();
		}
		return this.notaCalculadaProducaoCientifica;
	}

	public void setJustificativaProducaoCientifica(
			String justificativaProducaoCientifica) {
		this.justificativaProducaoCientifica = justificativaProducaoCientifica;
	}

	public String getJustificativaProducaoCientifica() {
		return justificativaProducaoCientifica;
	}

	public void setAvaliacao(Avaliacao avaliacao) {
		List<AvaliacaoPublicacao> avaliacoes = avaliacao
				.getAvaliacoesPublicacoes();
		this.avaliacoesPublicacoes = new ArrayList<AvaliacaoPublicacao>(
				avaliacoes.size());
		AvaliacaoPublicacao novo;
		for (AvaliacaoPublicacao avaliacaoPublicacao : avaliacoes) {
			novo = new AvaliacaoPublicacao();
			novo.setTituloPublicacao(avaliacaoPublicacao.getTituloPublicacao());
			novo.setExtratoQualis(avaliacaoPublicacao.getExtratoQualis());
			novo.setOrdemAutores(avaliacaoPublicacao.getOrdemAutores());
			novo.setTipoPublicacao(avaliacaoPublicacao.getTipoPublicacao());
			this.avaliacoesPublicacoes.add(novo);
		}
		this.justificativaProducaoCientifica = avaliacao
				.getJustificativaProducaoCientifica();
		this.notaSomatorioProducaoCientifica = avaliacao
				.getNotaSomatorioProducaoCientifica();
		this.notaCalculadaProducaoCientifica = avaliacao
				.getNotaCalculadaProducaoCientifica();

		if (avaliacao.getNotaProducaoCientificaEditada() != null) {
			this.notaEditada = avaliacao.getNotaProducaoCientificaEditada();
		} else {
			this.notaEditada = false;
		}
	}

	public boolean getNotaEditada() {
		return notaEditada;
	}

	public void setNotaEditada(boolean notaEditada) {
		this.notaEditada = notaEditada;
	}
	
	public BigDecimal calcularNota(
			List<AvaliacaoPublicacao> avaliacoesPublicacoes) {
		BigDecimal mipp;
		BigDecimal sipp;

		BigDecimal mipe;
		BigDecimal sipe;

		List<AvaliacaoPublicacao> avaliacoesPeriodicos = new ArrayList<AvaliacaoPublicacao>();
		List<AvaliacaoPublicacao> avaliacoesEventos = new ArrayList<AvaliacaoPublicacao>();

		for (AvaliacaoPublicacao avaliacao : avaliacoesPublicacoes) {
			if (avaliacao.getTipoPublicacao() == TipoPublicacao.PERIODICO) {
				avaliacoesPeriodicos.add(avaliacao);
			} else {
				avaliacoesEventos.add(avaliacao);
			}
		}

		Collections.sort(avaliacoesPeriodicos);
		Collections.sort(avaliacoesEventos);

		this.removerInvalidos(avaliacoesPeriodicos);
		this.removerInvalidos(avaliacoesEventos);

		if (!avaliacoesPeriodicos.isEmpty()) {
			mipp = avaliacoesPeriodicos.get(0).getValor();
			sipp = this.calcularSoma(avaliacoesPeriodicos);
		} else {
			mipp = new BigDecimal("0.0").setScale(Avaliacao.SCALE,
					Avaliacao.ROUND_MODE);
			sipp = new BigDecimal("0.0").setScale(Avaliacao.SCALE,
					Avaliacao.ROUND_MODE);
		}

		if (!avaliacoesEventos.isEmpty()) {
			mipe = avaliacoesEventos.get(0).getValor();
			sipe = this.calcularSoma(avaliacoesEventos);
		} else {
			mipe = new BigDecimal("0.0").setScale(Avaliacao.SCALE,
					Avaliacao.ROUND_MODE);
			sipe = new BigDecimal("0.0").setScale(Avaliacao.SCALE,
					Avaliacao.ROUND_MODE);
		}

		BigDecimal retorno = mipp.add(sipp).multiply(Avaliacao.DEZ).add(
				mipe.add(sipe).multiply(Avaliacao.CINCO)).setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
		return retorno;
	}

	private void removerInvalidos(List<AvaliacaoPublicacao> avaliacoes) {
		AvaliacaoPublicacao avaliacaoPublicacao;
		for (int i = avaliacoes.size() - 1; i >= 0; i--) {
			avaliacaoPublicacao = avaliacoes.get(i);
			if (avaliacaoPublicacao.getExtratoQualis() == null
					|| avaliacaoPublicacao.getOrdemAutores() == null
					|| avaliacaoPublicacao.getTipoPublicacao() == null) {
				avaliacoes.remove(i);
			}
		}
	}

	private BigDecimal calcularSoma(
			List<AvaliacaoPublicacao> avaliacoesPublicacoes) {
		BigDecimal retorno = new BigDecimal("0.0").setScale(Avaliacao.SCALE,
				Avaliacao.ROUND_MODE);

		for (ExtratoQualis qualis : ExtratoQualis.values()) {
			retorno = retorno.add(calcularExtratos(avaliacoesPublicacoes,
					qualis));
		}

		return retorno;
	}

	private BigDecimal calcularExtratos(List<AvaliacaoPublicacao> avaliacoes,
			ExtratoQualis extrato) {
		BigDecimal retorno = new BigDecimal("0.0").setScale(Avaliacao.SCALE,
				Avaliacao.ROUND_MODE);
		AvaliacaoPublicacao avaliacao;
		int expoente = 2;
		for (int i = 1; i < avaliacoes.size(); i++) {
			avaliacao = avaliacoes.get(i);
			if (avaliacao.getExtratoQualis() == extrato) {
				BigDecimal pow = avaliacao.getValor().pow(expoente);
				retorno = retorno.add(pow);
				expoente++;
			}
		}
		return retorno;
	}
}
