import java.math.BigDecimal;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.cfg.AnnotationConfiguration;

import com.improvess.selecao.model.Candidato;
import com.improvess.selecao.model.Doutorado;
import com.improvess.selecao.model.PosGraduacao;
import com.improvess.selecao.model.TipoPosGraduacao;
import com.improvess.selecpos.model.Avaliacao;
import com.improvess.selecpos.model.AvaliacaoDoutorado;
import com.improvess.selecpos.model.TipoCurso;

public class Main {
	public static SessionFactory factory;
	private static final BigDecimal PESO_HISTORICO_GRADUACAO = new BigDecimal(
			"5.5").setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
	private static final BigDecimal PESO_PRODUCAO_CIENTIFICA = new BigDecimal(
			"3.0").setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
	private static final BigDecimal PESO_EXPERIENCIA_DOCENTE = new BigDecimal(
			"0.5").setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
	private static final BigDecimal PESO_PEDEI = new BigDecimal("0.5")
			.setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
	private static final BigDecimal PESO_EXPERIENCIA_NAO_DOCENTE = new BigDecimal(
			"0.5").setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
	private static final BigDecimal DEZ = new BigDecimal("10.0");

	public static SessionFactory getInstance() {
		if (factory == null) {
			factory = new AnnotationConfiguration().configure()
					.buildSessionFactory();
		}
		return factory;
	}

	@SuppressWarnings("unchecked")
	public static void main(String[] args) {
		Session session = null;
		Transaction trans = null;
		try {
			session = getInstance().openSession();
			trans = session.beginTransaction();

			Query query = session
					.createQuery("From Candidato WHERE tipoCurso = 'DOUTORADO'");
			Iterator<Candidato> candidatos = query.iterate();
			DadosHistorico calculador = new DadosHistorico();
			AvaliacaoDoutorado avaliacao;
			while (candidatos.hasNext()) {
				Candidato candidato = candidatos.next();
				Doutorado doutorado = candidato.getDoutorado();
				avaliacao = doutorado.getAvaliacaoDoutoradoCVUm();
				if (avaliacao != null) {
					BigDecimal notaGraduacao = candidato.getGraduacao()
							.getMediaConvertida()
							.setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
					BigDecimal notaMestrado = Avaliacao.ZERO;
					PosGraduacao posGraduacao = candidato.getPosGraduacao();
					if (posGraduacao.getTipoCurso() == TipoPosGraduacao.MESTRADO) {
						notaMestrado = posGraduacao
								.getMediaConvertida()
								.setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
					}
					corrigir(notaGraduacao, notaMestrado, calculador,
							avaliacao, session);
					avaliacao = doutorado.getAvaliacaoDoutoradoCVDois();
					if (avaliacao != null) {
						corrigir(notaGraduacao, notaMestrado, calculador,
								avaliacao, session);
					}
				}
			}

			trans.commit();
		} catch (Exception e) {
			if (trans != null) {
				trans.rollback();
			}
			e.printStackTrace();
		} finally {
			if (session != null) {
				session.close();
			}
		}
	}

	private static void corrigir(BigDecimal notaGraduacao,
			BigDecimal notaMestrado, DadosHistorico calculadorHG,
			AvaliacaoDoutorado avaliacao, Session session) {

		boolean notaEditada = notaEditada(avaliacao, session,
				"notaHistoricoGraduacaoEditada");
		boolean temPremios = notaEditada(avaliacao, session, "temPremios");
		boolean temPosGraduacaoConcluida = notaEditada(avaliacao, session,
				"temPosGraduacaoConcluida");
		boolean conclusaoMestradoSuperiorLimite = notaEditada(avaliacao,
				session, "conclusaoMestradoSuperiorLimite");

		calculadorHG.reset(notaGraduacao);
		calculadorHG.setTipoCurso(TipoCurso.DOUTORADO);
		calculadorHG.setNotaMestrado(notaMestrado);
		calculadorHG.setAvaliacaoDoutorado(avaliacao);

		calculadorHG.setTemPremios(temPremios);
		calculadorHG.setTemPosGraduacaoConcluida(temPosGraduacaoConcluida);
		calculadorHG
				.setConclusaoMestradoSuperiorLimite(conclusaoMestradoSuperiorLimite);

		calculadorHG.setNotaEditada(notaEditada);

		BigDecimal notaCalculadaHistoricoGraduacao = calculadorHG
				.getNotaCalculadaHistoricoGraduacao().setScale(Avaliacao.SCALE,
						Avaliacao.ROUND_MODE);

		if (!notaEditada) {
			SQLQuery queryUpdateHG = session.createSQLQuery("UPDATE Avaliacao"
					+ " SET notaCalculadaHistoricoGraduacao = "
					+ notaCalculadaHistoricoGraduacao + " WHERE id = "
					+ avaliacao.getId());
			queryUpdateHG.executeUpdate();

			boolean notaFinalEditada = notaEditada(avaliacao, session,
					"notaFinalEditada");

			if (!notaFinalEditada) {
				BigDecimal notaFinal = calcularNotaFinalHG(avaliacao,
						calculadorHG, session);

				SQLQuery queryUpdateNotaFinal = session
						.createSQLQuery("UPDATE Avaliacao"
								+ " SET notaFinal = " + notaFinal
								+ " WHERE id = " + avaliacao.getId());
				queryUpdateNotaFinal.executeUpdate();
			}
		}
	}

	private static boolean notaEditada(AvaliacaoDoutorado avaliacao,
			Session session, String colunaNota) {
		SQLQuery queryNotaFinalEditada = session.createSQLQuery("SELECT "
				+ colunaNota + " FROM avaliacao" + " WHERE " + colunaNota
				+ " = 1 AND id = " + avaliacao.getId());
		boolean notaFinalEditada = !queryNotaFinalEditada.list().isEmpty();
		return notaFinalEditada;
	}

	private static BigDecimal calcularNotaFinalHG(AvaliacaoDoutorado avaliacao,
			DadosHistorico dadosHistorico, Session session) {
		BigDecimal notaHistoricoGraduacao = dadosHistorico
				.getNotaCalculadaHistoricoGraduacao();
		BigDecimal notaExperienciaDocente = avaliacao
				.getNotaCalculadaExperienciaDocencente();
		BigDecimal notaExperienciaPeDeI = avaliacao.getNotaCalculadaPeDeI();
		BigDecimal notaProfissionalNaoDocente = avaliacao
				.getNotaCalculadaExperienciaNaoDocente();
		BigDecimal notaProducaoCientifica = avaliacao
				.getNotaCalculadaProducaoCientifica();

		BigDecimal notaRelativaGraduacao = notaHistoricoGraduacao
				.multiply(PESO_HISTORICO_GRADUACAO);
		BigDecimal notaRelativaExperienciaDocente = notaExperienciaDocente
				.multiply(PESO_EXPERIENCIA_DOCENTE);
		BigDecimal notaRelativaExperieniaPeDeI = notaExperienciaPeDeI
				.multiply(PESO_PEDEI);
		BigDecimal notaRelativaProducaoCientifica = notaProducaoCientifica
				.multiply(PESO_PRODUCAO_CIENTIFICA);
		BigDecimal notaRelativaProfissionalNaoDocente = notaProfissionalNaoDocente
				.multiply(PESO_EXPERIENCIA_NAO_DOCENTE);

		return notaRelativaGraduacao.add(notaRelativaExperienciaDocente)
				.add(notaRelativaExperieniaPeDeI)
				.add(notaRelativaProducaoCientifica)
				.add(notaRelativaProfissionalNaoDocente).divide(DEZ)
				.setScale(Avaliacao.SCALE, Avaliacao.ROUND_MODE);
	}
}
