package com.improvess.selecao.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToOne;
import javax.persistence.Table;

import com.improvess.selecpos.model.TipoCurso;

@SuppressWarnings("serial")
@Entity
@Table(name = "Candidato")
public class Candidato implements Serializable {

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private Long id;

	@Column(name = "tipocurso")
	@Enumerated(EnumType.STRING)
	private TipoCurso tipoCurso = TipoCurso.MESTRADO;

	@Column(name = "nome", nullable = false, length = 255)
	private String nome;

	@Column(name = "identificacao")
	private Identificacao identificacao = Identificacao.CPF;

	@Column(name = "numeroIdentificacao")
	private String numeroIdentificacao;

	@Column(name = "sexo")
	private Sexo sexo = Sexo.MASCULINO;

	@Column(name = "dataNascimento")
	private Date dataNascimento;

	@Column(name = "estadoCivil")
	private EstadoCivil estadoCivil = EstadoCivil.SOLTEIRO;

	@Column(name = "mae")
	private String mae;

	@Column(name = "pai")
	private String pai;

	@OneToOne(cascade = CascadeType.ALL)
	private Graduacao graduacao;

	@OneToOne(cascade = CascadeType.ALL)
	private PosGraduacao posGraduacao;

	@OneToOne(cascade = CascadeType.ALL)
	private Mestrado mestrado;

	@OneToOne(cascade = CascadeType.ALL)
	private Doutorado doutorado;

	@OneToOne(cascade = CascadeType.ALL)
	private Origem origem;

	@OneToOne(cascade = CascadeType.ALL)
	private Identidade identidade;

	@OneToOne(cascade = CascadeType.ALL)
	private Endereco endereco;

	private TipoPosGraduacao tipoPosGraduacao;

	private Regime regime;

	@OneToOne(cascade = CascadeType.ALL)
	private DadosVinculoEmpregaticio dadosVinculoEmpregaticio;

	@OneToOne(cascade = CascadeType.ALL)
	private DadosConvenio dadosConvenio;

	private Boolean solicitaBolsa;

	@OneToOne(cascade = CascadeType.ALL)
	private Publicacoes publicacoes;

	private String numeroInscricao;

	@Override
	public boolean equals(Object obj) {
		boolean retorno = false;
		if (obj instanceof Candidato) {
			Candidato candidato = (Candidato) obj;
			retorno = this.identidade.getNumero() != null
					&& this.identidade.getNumero().equalsIgnoreCase(
							candidato.identidade.getNumero());
		} 
		return retorno;
	}

	@Override
	public int hashCode() {
		int retorno = 0;
		if (this.identidade.getNumero() != null) {
			retorno = this.identidade.getNumero().hashCode();
		}
		return retorno;
	}

	public TipoCurso getTipoCurso() {
		return tipoCurso;
	}

	public void setTipoCurso(TipoCurso tipoCurso) {
		this.tipoCurso = tipoCurso;
	}

	public String getNome() {
		return nome;
	}

	public void setNome(String nome) {
		this.nome = nome;
	}

	public Identificacao getIdentificacao() {
		return identificacao;
	}

	public void setIdentificacao(Identificacao identificacao) {
		this.identificacao = identificacao;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Long getId() {
		return id;
	}

	public Sexo getSexo() {
		return sexo;
	}

	public void setSexo(Sexo sexo) {
		this.sexo = sexo;
	}

	public Date getDataNascimento() {
		return dataNascimento;
	}

	public void setDataNascimento(Date dataNascimento) {
		this.dataNascimento = dataNascimento;
	}

	public EstadoCivil getEstadoCivil() {
		return estadoCivil;
	}

	public void setEstadoCivil(EstadoCivil estadoCivil) {
		this.estadoCivil = estadoCivil;
	}

	public void setNumeroIdentificacao(String numeroIdentificacao) {
		this.numeroIdentificacao = numeroIdentificacao;
	}

	public String getNumeroIdentificacao() {
		return numeroIdentificacao;
	}

	public void setOrigem(Origem origem) {
		this.origem = origem;
	}

	public Origem getOrigem() {
		return origem;
	}

	public void setIdentidade(Identidade identidade) {
		this.identidade = identidade;
	}

	public Identidade getIdentidade() {
		return identidade;
	}

	public String getMae() {
		return mae;
	}

	public void setMae(String mae) {
		this.mae = mae;
	}

	public String getPai() {
		return pai;
	}

	public void setPai(String pai) {
		this.pai = pai;
	}

	public Endereco getEndereco() {
		return endereco;
	}

	public void setEndereco(Endereco endereco) {
		this.endereco = endereco;
	}

	public void setGraduacao(Graduacao graduacao) {
		this.graduacao = graduacao;
	}

	public Graduacao getGraduacao() {
		return graduacao;
	}

	public void setPosGraduacao(PosGraduacao posGraduacao) {
		this.posGraduacao = posGraduacao;
	}

	public PosGraduacao getPosGraduacao() {
		return posGraduacao;
	}

	public void setMestrado(Mestrado mestrado) {
		this.mestrado = mestrado;
	}

	public Mestrado getMestrado() {
		return mestrado;
	}

	public void setTipoPosGraduacao(TipoPosGraduacao tipoPosGraduacao) {
		this.tipoPosGraduacao = tipoPosGraduacao;
	}

	public TipoPosGraduacao getTipoPosGraduacao() {
		return tipoPosGraduacao;
	}

	public void setDoutorado(Doutorado doutorado) {
		this.doutorado = doutorado;
	}

	public Doutorado getDoutorado() {
		return doutorado;
	}

	public Regime getRegime() {
		return regime;
	}

	public void setRegime(Regime regime) {
		this.regime = regime;
	}

	public DadosVinculoEmpregaticio getDadosVinculoEmpregaticio() {
		return dadosVinculoEmpregaticio;
	}

	public void setDadosVinculoEmpregaticio(
			DadosVinculoEmpregaticio dadosVinculoEmpregaticio) {
		this.dadosVinculoEmpregaticio = dadosVinculoEmpregaticio;
	}

	public DadosConvenio getDadosConvenio() {
		return dadosConvenio;
	}

	public void setDadosConvenio(DadosConvenio dadosConvenio) {
		this.dadosConvenio = dadosConvenio;
	}

	public Boolean getSolicitaBolsa() {
		return solicitaBolsa;
	}

	public void setSolicitaBolsa(Boolean solicitaBolsa) {
		this.solicitaBolsa = solicitaBolsa;
	}

	public Publicacoes getPublicacoes() {
		return publicacoes;
	}

	public void setPublicacoes(Publicacoes publicacoes) {
		this.publicacoes = publicacoes;
	}

	public String getNumeroInscricao() {
		return numeroInscricao;
	}

	public void setNumeroInscricao(String numeroInscricao) {
		this.numeroInscricao = numeroInscricao;
	}

	public Boolean getEstaAvaliadoCV() {

		Boolean retorno = false;
		if (this.tipoCurso == TipoCurso.MESTRADO) {
			if (this.getMestrado().getAvaliacaoMestradoCVUm() != null
					&& this.getMestrado().getAvaliacaoMestradoCVDois() != null) {
				retorno = true;
			}
		} else {
			if (this.getDoutorado().getAvaliacaoDoutoradoCVUm() != null
					&& this.getDoutorado().getAvaliacaoDoutoradoCVDois() != null) {
				retorno = true;
			}
		}
		return retorno;

	}

	public Long getAvaliadorUm() {
		Long retorno = null;

		if (this.getTipoCurso() == TipoCurso.MESTRADO) {
			if (this.getMestrado().getAvaliacaoMestradoCVUm() != null) {
				retorno = this.getMestrado().getAvaliacaoMestradoCVUm()
						.getAvaliador().getId();
			}
		} else {
			if (this.getDoutorado().getAvaliacaoDoutoradoCVUm() != null) {
				retorno = this.getDoutorado().getAvaliacaoDoutoradoCVUm()
						.getAvaliador().getId();
			}
		}
		return retorno;
	}

	public Long getAvaliadorDois() {
		Long retorno = null;

		if (this.getTipoCurso() == TipoCurso.MESTRADO) {
			if (this.getMestrado().getAvaliacaoMestradoCVDois() != null) {
				retorno = this.getMestrado().getAvaliacaoMestradoCVDois()
						.getAvaliador().getId();
			}
		} else {
			if (this.getDoutorado().getAvaliacaoDoutoradoCVDois() != null) {
				retorno = this.getDoutorado().getAvaliacaoDoutoradoCVDois()
						.getAvaliador().getId();
			}
		}
		return retorno;
	}
	
	public String getNotaCVUm() {
		String retorno = null;

		if (this.getTipoCurso() == TipoCurso.MESTRADO) {
			if (this.getMestrado().getAvaliacaoMestradoCVUm() != null) {
				retorno = this.getMestrado().getAvaliacaoMestradoCVUm()
						.getNotaFinal().toString();
			}
		} else {
			if (this.getDoutorado().getAvaliacaoDoutoradoCVUm() != null) {
				retorno = this.getDoutorado().getAvaliacaoDoutoradoCVUm()
						.getNotaFinal().toString();
			}
		}
		return retorno;
	}

	public String getNotaCVDois() {
		String retorno = null;

		if (this.getTipoCurso() == TipoCurso.MESTRADO) {
			if (this.getMestrado().getAvaliacaoMestradoCVDois() != null) {
				retorno = this.getMestrado().getAvaliacaoMestradoCVDois()
						.getNotaFinal().toString();
			}
		} else {
			if (this.getDoutorado().getAvaliacaoDoutoradoCVDois() != null) {
				retorno = this.getDoutorado().getAvaliacaoDoutoradoCVDois()
						.getNotaFinal().toString();
			}
		}
		return retorno;
	}

	public String getNomeInstituicaoGraduacao() {
		String retorno = "";
		if (this.graduacao.getInstituicao().getInstituicao().equals("Outro")) {
			retorno = this.graduacao.getOutraInstituicao();
		} else {
			retorno = this.graduacao.getInstituicao().getInstituicao();
		}
		return retorno;
	}

	public String getNomeCursoGraduacao() {
		String retorno = "";
		if (this.graduacao.getCurso().getCurso().equals("Outro")) {
			retorno = this.graduacao.getOutroCurso();
		} else {
			retorno = this.graduacao.getCurso().getCurso();
		}
		return retorno;
	}

	public String getNomeInstituicaoPosGraduacao() {
		String retorno = "";
		if (this.posGraduacao != null
				&& this.posGraduacao.getInstituicao() != null) {
			if (this.posGraduacao.getInstituicao().getInstituicao().equals(
					"Outro")) {
				retorno = this.posGraduacao.getOutraInstituicao();
			} else {
				retorno = this.posGraduacao.getInstituicao().getInstituicao();
			}
		}
		return retorno;
	}

	public String getNomeCursoPosGraduacao() {
		String retorno = "";
		if (this.posGraduacao != null && this.posGraduacao.getCurso() != null) {
			if (this.posGraduacao.getCurso().getCurso().equals("Outro")) {
				retorno = this.posGraduacao.getOutroCurso();
			} else {
				retorno = this.posGraduacao.getCurso().getCurso();
			}
		}
		return retorno;
	}
}
