package com.improvess.selecpos.model;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;

import com.improvess.selecao.model.Candidato;

@SuppressWarnings("serial")
@Entity
@Table(name = "avaliacao")
@Inheritance(strategy = InheritanceType.SINGLE_TABLE)
public abstract class Avaliacao implements Serializable {

	public static final int SCALE = 4;
	public static final int ROUND_MODE = BigDecimal.ROUND_DOWN;

	public static final BigDecimal DEZ = new BigDecimal("10.0").setScale(SCALE,
			ROUND_MODE);
	public static final BigDecimal NOVE = new BigDecimal("9.0").setScale(SCALE,
			ROUND_MODE);
	public static final BigDecimal OITO = new BigDecimal("8.0").setScale(SCALE,
			ROUND_MODE);
	public static final BigDecimal SETE = new BigDecimal("7.0").setScale(SCALE,
			ROUND_MODE);
	public static final BigDecimal SEIS = new BigDecimal("6.0").setScale(SCALE,
			ROUND_MODE);
	public static final BigDecimal CINCO = new BigDecimal("5.0").setScale(
			SCALE, ROUND_MODE);
	public static final BigDecimal QUATRO = new BigDecimal("4.0").setScale(
			SCALE, ROUND_MODE);
	public static final BigDecimal TRES = new BigDecimal("3.0").setScale(SCALE,
			ROUND_MODE);
	public static final BigDecimal ZERO_VIRGULA_CINCO = new BigDecimal("0.5")
			.setScale(SCALE, ROUND_MODE);
	public static final BigDecimal DOIS = new BigDecimal("2.0").setScale(SCALE,
			ROUND_MODE);
	public static final BigDecimal UM = new BigDecimal("1.0").setScale(SCALE,
			ROUND_MODE);
	public static final BigDecimal ZERO = new BigDecimal("0.0").setScale(SCALE,
			ROUND_MODE);
	public static final BigDecimal ZERO_OITO_CINCO = new BigDecimal("0.85")
			.setScale(SCALE, ROUND_MODE);
	public static final BigDecimal ZERO_SETE = new BigDecimal("0.7").setScale(
			SCALE, ROUND_MODE);
	public static final BigDecimal ZERO_CINCO = new BigDecimal("0.5").setScale(
			SCALE, ROUND_MODE);
	public static final BigDecimal ZERO_DOIS = new BigDecimal("0.2").setScale(
			SCALE, ROUND_MODE);
	public static final BigDecimal ZERO_UM = new BigDecimal("0.1").setScale(
			SCALE, ROUND_MODE);
	public static final BigDecimal ZERO_ZERO_CINCO = new BigDecimal("0.05")
			.setScale(SCALE, ROUND_MODE);

	public static final BigDecimal MEDIA_APROVACAO = OITO;
	public static final BigDecimal PESO_AVALIACAO_CV = OITO;
	public static final BigDecimal PESO_AVALIACAO_PP = DOIS;

	@Id
	@GeneratedValue(strategy = GenerationType.AUTO)
	private Long id;

	@OneToOne
	private Candidato candidato;

	@Enumerated(EnumType.STRING)
	private ConceitoCapes conceitoCapes;
	private Boolean temPremios;
	private Boolean temPosGraduacaoConcluida;
	
	@Column(length=1000)
	private String justificativaNotaHistoricoGraduacao;
	@Column(precision = 19, scale = Avaliacao.SCALE)
	private BigDecimal notaCalculadaHistoricoGraduacao;
	private Boolean notaHistoricoGraduacaoEditada;

	@OneToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
	private List<AvaliacaoPublicacao> avaliacoesPublicacoes;
	@Column(precision = 19, scale = Avaliacao.SCALE)
	private BigDecimal notaSomatorioProducaoCientifica;
	
	@Column(length=1000)
	private String justificativaProducaoCientifica;
	@Column(precision = 19, scale = Avaliacao.SCALE)
	private BigDecimal notaCalculadaProducaoCientifica;
	private Boolean notaProducaoCientificaEditada;

	@Enumerated(EnumType.STRING)
	private NotaConceitual terceiroGrauAreaAfim;
	@Enumerated(EnumType.STRING)
	private NotaConceitual monitoria;
	@Enumerated(EnumType.STRING)
	private NotaConceitual assistenteLaboratorio;
	
	@Column(length=1000)
	private String justificativaExperienciaDocente;
	@Column(precision = 19, scale = Avaliacao.SCALE)
	private BigDecimal notaSomatorioExperienciaDocencente;
	@Column(precision = 19, scale = Avaliacao.SCALE)
	private BigDecimal notaCalculadaExperienciaDocencente;
	private Boolean notaExperienciaDocencenteEditada;

	@Enumerated(EnumType.STRING)
	private NotaConceitual inciacaoCientifica;
	@Enumerated(EnumType.STRING)
	private NotaConceitual bolsaPeDeI;

	@Column(length=1000)
	private String justificativaPeDeI;
	@Column(precision = 19, scale = Avaliacao.SCALE)
	private BigDecimal notaSomatorioPeDeI;
	@Column(precision = 19, scale = Avaliacao.SCALE)
	private BigDecimal notaCalculadaPeDeI;
	private Boolean notaPeDeIEditada;

	@Enumerated(EnumType.STRING)
	private NotaConceitual superiorComVinculoEmpregaticioNaAreaCoodenando;
	@Enumerated(EnumType.STRING)
	private NotaConceitual superiorComVinculoEmpregaticioNaArea;
	@Enumerated(EnumType.STRING)
	private NotaConceitual medioComVinculoEmpregaticioNaArea;
	@Enumerated(EnumType.STRING)
	private NotaConceitual superiorSemVinculo;
	@Enumerated(EnumType.STRING)
	private NotaConceitual estagioExtraCurricular;
	@Enumerated(EnumType.STRING)
	private NotaConceitual profissionalAreaDistinta;

	@Column(length=1000)
	private String justificativaExperienciaNaoDocente;
	@Column(precision = 19, scale = Avaliacao.SCALE)
	private BigDecimal notaSomatorioExperienciaNaoDocente;
	@Column(precision = 19, scale = Avaliacao.SCALE)
	private BigDecimal notaCalculadaExperienciaNaoDocente;
	private Boolean notaExperienciaNaoDocenteEditada;

	@Column(precision = 19, scale = Avaliacao.SCALE)
	private BigDecimal notaFinal;
	private Boolean notaFinalEditada;
	@OneToOne
	private Docente avaliador;
	
	private Date dataAvaliacao;

	public Candidato getCandidato() {
		return candidato;
	}

	public void setCandidato(Candidato candidato) {
		this.candidato = candidato;
	}

	public ConceitoCapes getConceitoCapes() {
		return conceitoCapes;
	}

	public void setConceitoCapes(ConceitoCapes conceitoCapes) {
		this.conceitoCapes = conceitoCapes;
	}

	public Boolean getTemPremios() {
		return temPremios;
	}

	public void setTemPremios(Boolean temPremios) {
		this.temPremios = temPremios;
	}

	public Boolean getTemPosGraduacaoConcluida() {
		return temPosGraduacaoConcluida;
	}

	public void setTemPosGraduacaoConcluida(Boolean temPosGraduacaoConcluida) {
		this.temPosGraduacaoConcluida = temPosGraduacaoConcluida;
	}

	public String getJustificativaNotaHistoricoGraduacao() {
		return justificativaNotaHistoricoGraduacao;
	}

	public void setJustificativaNotaHistoricoGraduacao(
			String justificativaNotaHistoricoGraduacao) {
		this.justificativaNotaHistoricoGraduacao = justificativaNotaHistoricoGraduacao;
	}

	public BigDecimal getNotaCalculadaHistoricoGraduacao() {
		return notaCalculadaHistoricoGraduacao;
	}

	public void setNotaCalculadaHistoricoGraduacao(
			BigDecimal notaCalculadaHistoricoGraduacao) {
		this.notaCalculadaHistoricoGraduacao = notaCalculadaHistoricoGraduacao;
	}

	public Boolean getNotaHistoricoGraduacaoEditada() {
		return notaHistoricoGraduacaoEditada;
	}

	public void setNotaHistoricoGraduacaoEditada(
			Boolean notaHistoricoGraduacaoEditada) {
		this.notaHistoricoGraduacaoEditada = notaHistoricoGraduacaoEditada;
	}

	public List<AvaliacaoPublicacao> getAvaliacoesPublicacoes() {
		return avaliacoesPublicacoes;
	}

	public void setAvaliacoesPublicacoes(
			List<AvaliacaoPublicacao> avaliacoesPublicacoes) {
		this.avaliacoesPublicacoes = avaliacoesPublicacoes;
	}

	public BigDecimal getNotaSomatorioProducaoCientifica() {
		return notaSomatorioProducaoCientifica;
	}

	public void setNotaSomatorioProducaoCientifica(
			BigDecimal notaSomatorioProducaoCientifica) {
		this.notaSomatorioProducaoCientifica = notaSomatorioProducaoCientifica;
	}

	public BigDecimal getNotaCalculadaProducaoCientifica() {
		return notaCalculadaProducaoCientifica;
	}

	public void setNotaCalculadaProducaoCientifica(
			BigDecimal notaCalculadaProducaoCientifica) {
		this.notaCalculadaProducaoCientifica = notaCalculadaProducaoCientifica;
	}

	public String getJustificativaProducaoCientifica() {
		return justificativaProducaoCientifica;
	}

	public void setJustificativaProducaoCientifica(
			String justificativaProducaoCientifica) {
		this.justificativaProducaoCientifica = justificativaProducaoCientifica;
	}

	public NotaConceitual getTerceiroGrauAreaAfim() {
		return terceiroGrauAreaAfim;
	}

	public void setTerceiroGrauAreaAfim(NotaConceitual terceiroGrauAreaAfim) {
		this.terceiroGrauAreaAfim = terceiroGrauAreaAfim;
	}

	public NotaConceitual getMonitoria() {
		return monitoria;
	}

	public void setMonitoria(NotaConceitual monitoria) {
		this.monitoria = monitoria;
	}

	public NotaConceitual getAssistenteLaboratorio() {
		return assistenteLaboratorio;
	}

	public void setAssistenteLaboratorio(NotaConceitual assistenteLaboratorio) {
		this.assistenteLaboratorio = assistenteLaboratorio;
	}

	public String getJustificativaExperienciaDocente() {
		return justificativaExperienciaDocente;
	}

	public void setJustificativaExperienciaDocente(
			String justificativaExperienciaDocente) {
		this.justificativaExperienciaDocente = justificativaExperienciaDocente;
	}

	public BigDecimal getNotaSomatorioExperienciaDocencente() {
		return notaSomatorioExperienciaDocencente;
	}

	public void setNotaSomatorioExperienciaDocencente(
			BigDecimal notaSomatorioExperienciaDocencente) {
		this.notaSomatorioExperienciaDocencente = notaSomatorioExperienciaDocencente;
	}

	public BigDecimal getNotaCalculadaExperienciaDocencente() {
		return notaCalculadaExperienciaDocencente;
	}

	public void setNotaCalculadaExperienciaDocencente(
			BigDecimal notaCalculadaExperienciaDocencente) {
		this.notaCalculadaExperienciaDocencente = notaCalculadaExperienciaDocencente;
	}

	public NotaConceitual getInciacaoCientifica() {
		return inciacaoCientifica;
	}

	public void setInciacaoCientifica(NotaConceitual inciacaoCientifica) {
		this.inciacaoCientifica = inciacaoCientifica;
	}

	public String getJustificativaPeDeI() {
		return justificativaPeDeI;
	}

	public void setJustificativaPeDeI(String justificativaPeDeI) {
		this.justificativaPeDeI = justificativaPeDeI;
	}

	public NotaConceitual getBolsaPeDeI() {
		return bolsaPeDeI;
	}

	public void setBolsaPeDeI(NotaConceitual bolsaPeDeI) {
		this.bolsaPeDeI = bolsaPeDeI;
	}

	public BigDecimal getNotaSomatorioPeDeI() {
		return notaSomatorioPeDeI;
	}

	public void setNotaSomatorioPeDeI(BigDecimal notaSomatorioPeDeI) {
		this.notaSomatorioPeDeI = notaSomatorioPeDeI;
	}

	public BigDecimal getNotaCalculadaPeDeI() {
		return notaCalculadaPeDeI;
	}

	public void setNotaCalculadaPeDeI(BigDecimal notaCalculadaPeDeI) {
		this.notaCalculadaPeDeI = notaCalculadaPeDeI;
	}

	public NotaConceitual getSuperiorComVinculoEmpregaticioNaAreaCoodenando() {
		return superiorComVinculoEmpregaticioNaAreaCoodenando;
	}

	public void setSuperiorComVinculoEmpregaticioNaAreaCoodenando(
			NotaConceitual superiorComVinculoEmpregaticioNaAreaCoodenando) {
		this.superiorComVinculoEmpregaticioNaAreaCoodenando = superiorComVinculoEmpregaticioNaAreaCoodenando;
	}

	public NotaConceitual getSuperiorComVinculoEmpregaticioNaArea() {
		return superiorComVinculoEmpregaticioNaArea;
	}

	public void setSuperiorComVinculoEmpregaticioNaArea(
			NotaConceitual superiorComVinculoEmpregaticioNaArea) {
		this.superiorComVinculoEmpregaticioNaArea = superiorComVinculoEmpregaticioNaArea;
	}

	public NotaConceitual getMedioComVinculoEmpregaticioNaArea() {
		return medioComVinculoEmpregaticioNaArea;
	}

	public void setMedioComVinculoEmpregaticioNaArea(
			NotaConceitual medioComVinculoEmpregaticioNaArea) {
		this.medioComVinculoEmpregaticioNaArea = medioComVinculoEmpregaticioNaArea;
	}

	public NotaConceitual getSuperiorSemVinculo() {
		return superiorSemVinculo;
	}

	public void setSuperiorSemVinculo(NotaConceitual superiorSemVinculo) {
		this.superiorSemVinculo = superiorSemVinculo;
	}

	public NotaConceitual getEstagioExtraCurricular() {
		return estagioExtraCurricular;
	}

	public void setEstagioExtraCurricular(NotaConceitual estagioExtraCurricular) {
		this.estagioExtraCurricular = estagioExtraCurricular;
	}

	public NotaConceitual getProfissionalAreaDistinta() {
		return profissionalAreaDistinta;
	}

	public void setProfissionalAreaDistinta(
			NotaConceitual profissionalAreaDistinta) {
		this.profissionalAreaDistinta = profissionalAreaDistinta;
	}

	public String getJustificativaExperienciaNaoDocente() {
		return justificativaExperienciaNaoDocente;
	}

	public void setJustificativaExperienciaNaoDocente(
			String justificativaExperienciaNaoDocente) {
		this.justificativaExperienciaNaoDocente = justificativaExperienciaNaoDocente;
	}

	public BigDecimal getNotaSomatorioExperienciaNaoDocente() {
		return notaSomatorioExperienciaNaoDocente;
	}

	public void setNotaSomatorioExperienciaNaoDocente(
			BigDecimal notaSomatorioExperienciaNaoDocente) {
		this.notaSomatorioExperienciaNaoDocente = notaSomatorioExperienciaNaoDocente;
	}

	public BigDecimal getNotaCalculadaExperienciaNaoDocente() {
		return notaCalculadaExperienciaNaoDocente;
	}

	public void setNotaCalculadaExperienciaNaoDocente(
			BigDecimal notaCalculadaExperienciaNaoDocente) {
		this.notaCalculadaExperienciaNaoDocente = notaCalculadaExperienciaNaoDocente;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public void setNotaFinal(BigDecimal notaFinal) {
		this.notaFinal = notaFinal;
	}

	public BigDecimal getNotaFinal() {
		return notaFinal;
	}

	public void setAvaliador(Docente avaliador) {
		this.avaliador = avaliador;
	}

	public Docente getAvaliador() {
		return avaliador;
	}

	public Boolean getNotaPeDeIEditada() {
		return notaPeDeIEditada;
	}

	public void setNotaPeDeIEditada(Boolean notaCalculadaPeDeIEditada) {
		this.notaPeDeIEditada = notaCalculadaPeDeIEditada;
	}

	public Boolean getNotaExperienciaDocencenteEditada() {
		return notaExperienciaDocencenteEditada;
	}

	public void setNotaExperienciaDocencenteEditada(
			Boolean notaExperienciaDocencenteEditada) {
		this.notaExperienciaDocencenteEditada = notaExperienciaDocencenteEditada;
	}

	public Boolean getNotaExperienciaNaoDocenteEditada() {
		return notaExperienciaNaoDocenteEditada;
	}

	public void setNotaExperienciaNaoDocenteEditada(
			Boolean notaExperienciaNaoDocenteEditada) {
		this.notaExperienciaNaoDocenteEditada = notaExperienciaNaoDocenteEditada;
	}

	public Boolean getNotaProducaoCientificaEditada() {
		return notaProducaoCientificaEditada;
	}

	public void setNotaProducaoCientificaEditada(
			Boolean notaProducaoCientificaEditada) {
		this.notaProducaoCientificaEditada = notaProducaoCientificaEditada;
	}
	
	public Boolean getNotaFinalEditada() {
		return notaFinalEditada;
	}

	public void setNotaFinalEditada(Boolean notaFinalEditada) {
		this.notaFinalEditada = notaFinalEditada;
	}

	public void setDataAvaliacao(Date dataAvaliacao) {
		this.dataAvaliacao = dataAvaliacao;
	}

	public Date getDataAvaliacao() {
		return dataAvaliacao;
	}
}
